@@ -134,6 +134,10 @@ namespace Meebey.SmartIrc4net
                 lock (this) {
                     _IsConnectionError = value;
                 }
+                if (value) {
+                    // signal ReadLine() to check IsConnectionError state
+                    _ReadThread.QueuedEvent.Set();
+                }
             }
         }
 
@@ -766,7 +770,6 @@ namespace Meebey.SmartIrc4net
             Logger.Connection.Info("reconnecting...");
 #endif
             Disconnect();
-            Thread.Sleep(AutoRetryDelay * 1000);
             Connect(_AddressList, _Port);
         }
         
@@ -863,7 +866,7 @@ namespace Meebey.SmartIrc4net
                 while (IsConnected &&
                        !IsConnectionError &&
                        _ReadThread.Queue.Count == 0) {
-                    Thread.Sleep(10);
+                    _ReadThread.QueuedEvent.WaitOne();
                 }
             }
 
@@ -905,6 +908,7 @@ namespace Meebey.SmartIrc4net
                 _WriteLine(data);
             } else {
                 ((Queue)_SendBuffer[priority]).Enqueue(data);
+                _WriteThread.QueuedEvent.Set();
             }
         }
 
@@ -921,8 +925,10 @@ namespace Meebey.SmartIrc4net
         {
             if (IsConnected) {
                 try {
-                    _Writer.Write(data + "\r\n");
-                    _Writer.Flush();
+                    lock (_Writer) {
+                        _Writer.Write(data + "\r\n");
+                        _Writer.Flush();
+                    }
                 } catch (IOException) {
 #if LOG4NET
                     Logger.Socket.Warn("sending data failed, connection lost");
@@ -1014,6 +1020,8 @@ namespace Meebey.SmartIrc4net
         {
             try {
                 if (AutoReconnect) {
+                    // prevent connect -> exception -> connect flood loop
+                    Thread.Sleep(AutoRetryDelay * 1000);
                     // lets try to recover the connection
                     Reconnect();
                 } else {
@@ -1036,6 +1044,8 @@ namespace Meebey.SmartIrc4net
             private Thread         _Thread;
             private Queue          _Queue = Queue.Synchronized(new Queue());
 
+            public AutoResetEvent  QueuedEvent;
+
             public Queue Queue {
                 get {
                     return _Queue;
@@ -1049,6 +1059,7 @@ namespace Meebey.SmartIrc4net
             public ReadThread(IrcConnection connection)
             {
                 _Connection = connection;
+                QueuedEvent = new AutoResetEvent(false);
             }
 
             /// <summary>
@@ -1102,6 +1113,7 @@ namespace Meebey.SmartIrc4net
                         while (_Connection.IsConnected &&
                                ((data = _Connection._Reader.ReadLine()) != null)) {
                             _Queue.Enqueue(data);
+                            QueuedEvent.Set();
 #if LOG4NET
                             Logger.Socket.Debug("received: \""+data+"\"");
 #endif
@@ -1153,6 +1165,8 @@ namespace Meebey.SmartIrc4net
             private int            _BelowMediumThresholdCount = 1;
             private int            _BurstCount;
 
+            public AutoResetEvent QueuedEvent;
+
             /// <summary>
             /// 
             /// </summary>
@@ -1160,6 +1174,7 @@ namespace Meebey.SmartIrc4net
             public WriteThread(IrcConnection connection)
             {
                 _Connection = connection;
+                QueuedEvent = new AutoResetEvent(false);
             }
 
             /// <summary>
@@ -1201,8 +1216,12 @@ namespace Meebey.SmartIrc4net
                 try {
                     try {
                         while (_Connection.IsConnected) {
-                            _CheckBuffer();
-                            Thread.Sleep(_Connection._SendDelay);
+                            QueuedEvent.WaitOne();
+                            var isBufferEmpty = false;
+                            do {
+                                isBufferEmpty = _CheckBuffer() == 0;
+                                Thread.Sleep(_Connection._SendDelay);
+                            } while (!isBufferEmpty);
                         }
                     } catch (IOException e) {
 #if LOG4NET
@@ -1232,19 +1251,25 @@ namespace Meebey.SmartIrc4net
 
 #region WARNING: complex scheduler, don't even think about changing it!
             // WARNING: complex scheduler, don't even think about changing it!
-            private void _CheckBuffer()
+            private int _CheckBuffer()
             {
-                // only send data if we are succefully registered on the IRC network
-                if (!_Connection._IsRegistered) {
-                    return;
-                }
-
                 _HighCount        = ((Queue)_Connection._SendBuffer[Priority.High]).Count;
                 _AboveMediumCount = ((Queue)_Connection._SendBuffer[Priority.AboveMedium]).Count;
                 _MediumCount      = ((Queue)_Connection._SendBuffer[Priority.Medium]).Count;
                 _BelowMediumCount = ((Queue)_Connection._SendBuffer[Priority.BelowMedium]).Count;
                 _LowCount         = ((Queue)_Connection._SendBuffer[Priority.Low]).Count;
 
+                var msgCount = _HighCount +
+                               _AboveMediumCount +
+                               _MediumCount +
+                               _BelowMediumCount +
+                               _LowCount;
+
+                // only send data if we are succefully registered on the IRC network
+                if (!_Connection._IsRegistered) {
+                    return msgCount;
+                }
+
                 if (_CheckHighBuffer() &&
                     _CheckAboveMediumBuffer() &&
                     _CheckMediumBuffer() &&
@@ -1261,6 +1286,8 @@ namespace Meebey.SmartIrc4net
                     _BurstCount++;
                     //_CheckBuffer();
                 }
+
+                return msgCount;
             }
 
             private bool _CheckHighBuffer()
@@ -1272,6 +1299,7 @@ namespace Meebey.SmartIrc4net
                         Logger.Queue.Warn("Sending data was not sucessful, data is requeued!");
 #endif
                         ((Queue)_Connection._SendBuffer[Priority.High]).Enqueue(data);
+                        return false;
                     }
 
                     if (_HighCount > 1) {
@@ -1293,6 +1321,7 @@ namespace Meebey.SmartIrc4net
                         Logger.Queue.Warn("Sending data was not sucessful, data is requeued!");
 #endif
                         ((Queue)_Connection._SendBuffer[Priority.AboveMedium]).Enqueue(data);
+                        return false;
                     }
                     _AboveMediumSentCount++;
 
@@ -1314,6 +1343,7 @@ namespace Meebey.SmartIrc4net
                         Logger.Queue.Warn("Sending data was not sucessful, data is requeued!");
 #endif
                         ((Queue)_Connection._SendBuffer[Priority.Medium]).Enqueue(data);
+                        return false;
                     }
                     _MediumSentCount++;
 
@@ -1335,6 +1365,7 @@ namespace Meebey.SmartIrc4net
                         Logger.Queue.Warn("Sending data was not sucessful, data is requeued!");
 #endif
                         ((Queue)_Connection._SendBuffer[Priority.BelowMedium]).Enqueue(data);
+                        return false;
                     }
                     _BelowMediumSentCount++;
 
@@ -1362,6 +1393,7 @@ namespace Meebey.SmartIrc4net
                         Logger.Queue.Warn("Sending data was not sucessful, data is requeued!");
 #endif
                         ((Queue)_Connection._SendBuffer[Priority.Low]).Enqueue(data);
+                        return false;
                     }
 
                     if (_LowCount > 1) {
@@ -1422,7 +1454,7 @@ namespace Meebey.SmartIrc4net
 #endif
                 try {
                     while (_Connection.IsConnected ) {
-                        Thread.Sleep(_Connection._IdleWorkerInterval);
+                        Thread.Sleep(_Connection._IdleWorkerInterval * 1000);
                         
                         // only send active pings if we are registered
                         if (!_Connection.IsRegistered) {
