@@ -26,11 +26,11 @@ namespace Smuxi.Common
 {
     public static class Defines
     {
-        public const string GitBranch   = "release/0.8.10.1210";
-        public const string GitCommitHash = "ebe7fd8";
+        public const string GitBranch   = "";
+        public const string GitCommitHash = "";
 
         private static readonly string f_InstallPrefix = "/usr/local";
-        private static readonly string f_DistVersion = "release/0.8.10.1210/ebe7fd8";
+        private static readonly string f_DistVersion = "dist-osx";
         private static readonly string f_TwitterApiKey = "60QV2qQx9cS7y1BJDbgAA|2VgD6qQKddsF5HYQ0TrRgs3tFTnCwDONBmRlTmG658";
 
         public static string InstallPrefix {
