@@ -1273,7 +1273,6 @@ namespace Smuxi.Engine
                 var userIds = new TwitterIdCollection(idPage);
                 var lookupOptions = CreateOptions<LookupUsersOptions>();
                 lookupOptions.UserIds = userIds;
-                lookupOptions.CacheOutput = true;
                 var lookupResponse = TwitterUser.Lookup(f_OAuthTokens, lookupOptions);
                 CheckResponse(lookupResponse);
                 var friends = lookupResponse.ResponseObject;
@@ -1562,11 +1561,11 @@ namespace Smuxi.Engine
             return person;
         }
 
-        protected override MessageBuilder CreateMessageBuilder()
+        protected override T CreateMessageBuilder<T>()
         {
             var builder = new TwitterMessageBuilder();
             builder.ApplyConfig(Session.UserConfig);
-            return builder;
+            return (T)(object) builder;
         }
 
         private static bool ValidateCertificate(object sender,
