@@ -114,8 +114,10 @@ namespace Smuxi.Engine
         
         protected T Get<T>(string key, T defaultvalue)
         {
+#if CONFIG_DEBUG
             Trace.Call(key, defaultvalue);
-            
+#endif
+
             string inisection = _IniGetSection(key);
             string inikey = _IniGetKey(key);
             IniSection section = m_IniDocument.Sections[inisection];
@@ -146,7 +148,9 @@ namespace Smuxi.Engine
 
         protected object Get(string key, object defaultvalue)
         {
+#if CONFIG_DEBUG
             Trace.Call(key, defaultvalue);
+#endif
 
 #if CONFIG_GCONF
             try {
@@ -201,8 +205,10 @@ namespace Smuxi.Engine
         
         private void _Set(string key, object valueobj)
         {
+#if CONFIG_DEBUG
             Trace.Call(key, valueobj);
-            
+#endif
+
 #if CONFIG_GCONF
             _GConf.Set(_GConfPrefix+key, valueobj);
 #elif CONFIG_NINI
@@ -308,6 +314,7 @@ namespace Smuxi.Engine
             Get(prefix + "Servers", new string[] {
                 "IRC/irc.oftc.net",
                 "IRC/irc.gimp.org",
+                "IRC/irc.geekshed.net",
                 "IRC/irc.efnet.org",
                 "IRC/irc.ircnet.org",
                 "IRC/irc.freenode.net"
@@ -337,6 +344,15 @@ namespace Smuxi.Engine
             Get(prefix + "UseEncryption", false);
             Get(prefix + "ValidateServerCertificate", false);
 
+            prefix = "Engine/Users/DEFAULT/Servers/IRC/irc.geekshed.net/";
+            Get(prefix + "Hostname", "irc.geekshed.net");
+            Get(prefix + "Port", 6667);
+            Get(prefix + "Network", "GeekShed");
+            Get(prefix + "Username", String.Empty);
+            Get(prefix + "Password", String.Empty);
+            Get(prefix + "UseEncryption", false);
+            Get(prefix + "ValidateServerCertificate", false);
+
             prefix = "Engine/Users/DEFAULT/Servers/IRC/irc.efnet.org/";
             Get(prefix + "Hostname", "irc.efnet.org");
             Get(prefix + "Port", 6667);
@@ -551,6 +567,10 @@ namespace Smuxi.Engine
                     } else {
                         m_Preferences[sprefix + "OnConnectCommands"] = commands;
                     }
+                    // HACK: XMPP specific values
+                    LoadEntry(sprefix+"Resource", null);
+                    LoadEntry(sprefix+"PriorityAvailable", null);
+                    LoadEntry(sprefix+"PriorityAway", null);
                 }
 
                 string[] filters = null;
@@ -609,8 +629,10 @@ namespace Smuxi.Engine
         
         public void Remove(string key)
         {
+#if CONFIG_DEBUG
             Trace.Call(key);
-            
+#endif
+
             bool isSection = false;
             if (key.EndsWith("/")) {
                 isSection = true;
@@ -645,8 +667,10 @@ namespace Smuxi.Engine
 
         protected void LoadUserEntry(string user, string key, object defaultvalue)
         {
+#if CONFIG_DEBUG
             Trace.Call(user, key, defaultvalue);
-            
+#endif
+
             string prefix = "Engine/Users/";
             string ukey = prefix+user+"/"+key;
             object obj;
@@ -662,8 +686,10 @@ namespace Smuxi.Engine
         
         protected void LoadEntry(string key, object defaultvalue)
         {
+#if CONFIG_DEBUG
             Trace.Call(key, defaultvalue);
-            
+#endif
+
             object obj;
             if (defaultvalue is string) {
                 obj = Get<string>(key, (string) defaultvalue);
