@@ -1,7 +1,7 @@
 /*
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2005-2012 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2005-2013 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -212,13 +212,12 @@ namespace Smuxi.Engine
                         isError = true;
                     }
                     if (isError) {
-                        fm.AddTextToChat(
-                            _SessionChat,
-                            String.Format(
-                                _("Automatic connect to {0} failed!"),
-                                server.Hostname + ":" + server.Port
-                            )
+                        var builder = CreateMessageBuilder();
+                        builder.AppendErrorText(
+                            _("Automatic connect to {0} failed!"),
+                            server.Hostname + ":" + server.Port
                         );
+                        fm.AddMessageToChat(_SessionChat, builder.ToMessage());
                         continue;
                     }
                 }
@@ -424,7 +423,9 @@ namespace Smuxi.Engine
                 "network list",
                 "network close [network]",
                 "network switch [network]",
-                "config (save|load)",
+                "config (save|load|list)",
+                "config get key",
+                "config set key=value",
                 "shutdown"
             };
 
@@ -445,7 +446,7 @@ namespace Smuxi.Engine
             }
             
             FrontendManager fm = cd.FrontendManager;
-
+            MessageBuilder builder = null;
             string protocol = null;
             ServerModel server = null;
             // first lookup by network name
@@ -496,10 +497,10 @@ namespace Smuxi.Engine
                         throw;
                     }
                     // this is an unknown protocol error
-                    fm.AddTextToChat(
-                        fm.CurrentChat,
-                        String.Format("-!- {0}", ex.Message)
-                    );
+                    builder = CreateMessageBuilder();
+                    builder.AppendEventPrefix();
+                    builder.AppendErrorText(ex.Message);
+                    fm.AddMessageToChat(cd.Chat, builder.ToMessage());
                     return;
                 }
             }
@@ -536,7 +537,10 @@ namespace Smuxi.Engine
 #if LOG4NET
                     f_Logger.Error("CommandConnect(): ", ex);
 #endif
-                    fm.AddTextToChat(cd.Chat, "-!- " + _("Connect failed!"));
+                    builder = CreateMessageBuilder();
+                    builder.AppendEventPrefix();
+                    builder.AppendErrorText(_("Connect failed!"));
+                    fm.AddMessageToChat(cd.Chat, builder.ToMessage());
                 }
             });
         }
@@ -555,14 +559,13 @@ namespace Smuxi.Engine
                 string server = cd.DataArray[1];
                 victim = GetProtocolManagerByHost(server);
                 if (victim == null) {
-                    fm.AddTextToChat(
-                        cd.Chat,
-                        "-!- " +
-                        String.Format(
-                            _("Disconnect failed - could not find server: {0}"),
-                            server
-                        )
+                    var builder = CreateMessageBuilder();
+                    builder.AppendEventPrefix();
+                    builder.AppendErrorText(
+                        _("Disconnect failed - could not find server: {0}"),
+                        server
                     );
+                    fm.AddMessageToChat(cd.Chat, builder.ToMessage());
                     return;
                 }
             } else {
@@ -597,8 +600,11 @@ namespace Smuxi.Engine
 #if LOG4NET
                     f_Logger.Error("CommandReconnect(): ", ex);
 #endif
-                    cd.FrontendManager.AddTextToChat(cd.Chat, "-!- " +
-                        _("Reconnect failed!"));
+                    var builder = CreateMessageBuilder();
+                    builder.AppendEventPrefix();
+                    builder.AppendErrorText(_("Reconnect failed!"));
+                    cd.FrontendManager.AddMessageToChat(cd.Chat,
+                                                        builder.ToMessage());
                 }
             });
         }
@@ -612,26 +618,91 @@ namespace Smuxi.Engine
             }
             
             FrontendManager fm = cd.FrontendManager;
-            if (cd.DataArray.Length >= 2) {
-                switch (cd.DataArray[1].ToLower()) {
-                    case "load":
-                        _Config.Load();
-                        fm.AddTextToChat(cd.Chat, "-!- " +
-                            _("Configuration reloaded"));
-                        break;
-                    case "save":
-                        _Config.Save();
-                        fm.AddTextToChat(cd.Chat, "-!- " +
-                            _("Configuration saved"));
-                        break;
-                    default:
-                        fm.AddTextToChat(cd.Chat, "-!- " + 
-                            _("Invalid parameter for config; use load or save"));
-                        break;
-                }
-            } else {
+            if (cd.DataArray.Length < 2) {
                 _NotEnoughParameters(cd);
+                return;
             }
+            var builder = CreateMessageBuilder();
+            builder.AppendEventPrefix();
+            var action = cd.DataArray[1].ToLower();
+            switch (action) {
+                case "load":
+                    _Config.Load();
+                    builder.AppendText(_("Configuration reloaded"));
+                    break;
+                case "save":
+                    _Config.Save();
+                    builder.AppendText(_("Configuration saved"));
+                    break;
+                case "get":
+                case "list":
+                    string key = null;
+                    if (action == "get") {
+                        if (cd.DataArray.Length < 3) {
+                            _NotEnoughParameters(cd);
+                            return;
+                        }
+                        key = cd.DataArray[2];
+                    }
+                    foreach (var entry in _UserConfig.OrderBy(kvp => kvp.Key)) {
+                        if (key != null &&
+                            entry.Key.IndexOf(key, StringComparison.InvariantCultureIgnoreCase) == -1) {
+                            continue;
+                        }
+                        builder = CreateMessageBuilder();
+                        builder.AppendEventPrefix();
+                        builder.AppendText("{0} = {1}", entry.Key, entry.Value);
+                        fm.AddMessageToChat(cd.Chat, builder.ToMessage());
+                    }
+                    return;
+                case "set":
+                    if (cd.DataArray.Length < 3) {
+                        _NotEnoughParameters(cd);
+                        return;
+                    }
+                    string setParam = cd.DataArray[2];
+                    if (!setParam.Contains("=")) {
+                        builder.AppendErrorText(
+                            _("Invalid key/value format.")
+                        );
+                        fm.AddMessageToChat(cd.Chat, builder.ToMessage());
+                        return;
+                    }
+                    string setKey = setParam.Split('=')[0];
+                    string setValue = setParam.Split('=')[1];
+                    object oldValue = _UserConfig[setKey];
+                    if (oldValue == null) {
+                        builder.AppendErrorText(
+                            _("Invalid config key: '{0}'"),
+                            setKey
+                        );
+                    } else {
+                        try {
+                            object newValue = Convert.ChangeType(setValue, oldValue.GetType());
+                            _UserConfig[setKey] = newValue;
+                            builder.AppendText("{0} = {1}", setKey, newValue.ToString());
+                        } catch (InvalidCastException) {
+                            builder.AppendErrorText(
+                                _("Could not convert config value: '{0}' to type: {1}"),
+                                setValue,
+                                oldValue.GetType().Name
+                            );
+                        } catch (FormatException) {
+                            builder.AppendErrorText(
+                                _("Could not convert config value: '{0}' to type: {1}"),
+                                setValue,
+                                oldValue.GetType().Name
+                            );
+                        }
+                    }
+                    break;
+                default:
+                    builder.AppendErrorText(
+                        _("Invalid parameter for config; use load, save, get or set.")
+                    );
+                    break;
+            }
+            fm.AddMessageToChat(cd.Chat, builder.ToMessage());
         }
         
         public void CommandShutdown(CommandModel cmd)
@@ -690,8 +761,13 @@ namespace Smuxi.Engine
                         _CommandNetworkClose(cd);
                         break;
                     default:
-                        fm.AddTextToChat(cd.Chat, "-!- " + 
-                            _("Invalid parameter for network; use list, switch, or close"));
+                        var builder = CreateMessageBuilder();
+                        builder.AppendEventPrefix();
+                        builder.AppendText(
+                            _("Invalid parameter for network; use list, " +
+                              "switch, or close")
+                        );
+                        fm.AddMessageToChat(cd.Chat, builder.ToMessage());
                         break;
                 }
             } else {
@@ -768,9 +844,10 @@ namespace Smuxi.Engine
                 string network = cd.DataArray[2];
                 pm = GetProtocolManagerByNetwork(network);
                 if (pm == null) {
-                    fm.AddTextToChat(cd.Chat, "-!- " +
-                        String.Format(_("Network close failed - could not find network: {0}"),
-                                      network));
+                    var builder = CreateMessageBuilder();
+                    builder.AppendText(_("Network close failed - could not " +
+                                         "find network: {0}"), network);
+                    fm.AddMessageToChat(cd.Chat, builder.ToMessage());
                     return;
                 }
             } else if (cd.DataArray.Length >= 2) {
@@ -789,7 +866,6 @@ namespace Smuxi.Engine
                     pm.Dispose();
                     // Dispose() takes care of removing the chat from session (frontends)
                     _ProtocolManagers.Remove(pm);
-                    fm.NextProtocolManager();
                 } catch (Exception ex) {
 #if LOG4NET
                     f_Logger.Error("_CommandNetworkClose(): Exception", ex);
@@ -806,10 +882,10 @@ namespace Smuxi.Engine
                 string network = cd.DataArray[2];
                 var pm = GetProtocolManagerByNetwork(network);
                 if (pm == null) {
-                    fm.AddTextToChat(cd.Chat, "-!- " +
-                        String.Format(
-                            _("Network switch failed - could not find network: {0}"),
-                                  network));
+                    var builder = CreateMessageBuilder();
+                    builder.AppendText(_("Network switch failed - could not " +
+                                         "find network: {0}"), network);
+                    fm.AddMessageToChat(cd.Chat, builder.ToMessage());
                     return;
                 }
                 fm.CurrentProtocolManager = pm;
@@ -824,25 +900,17 @@ namespace Smuxi.Engine
         
         private void _NotConnected(CommandModel cd)
         {
-            cd.FrontendManager.AddTextToChat(
-                cd.Chat,
-                String.Format("-!- {0}",
-                    _("Not connected to any network")
-                )
-            );
+            var builder = CreateMessageBuilder();
+            builder.AppendText(_("Not connected to any network"));
+            cd.FrontendManager.AddMessageToChat(cd.Chat, builder.ToMessage());
         }
         
         private void _NotEnoughParameters(CommandModel cd)
         {
-            cd.FrontendManager.AddTextToChat(
-                cd.Chat,
-                String.Format("-!- {0}",
-                    String.Format(
-                        _("Not enough parameters for {0} command"),
-                        cd.Command
-                    )
-                )
-            );
+            var builder = CreateMessageBuilder();
+            builder.AppendText(_("Not enough parameters for {0} command"),
+                               cd.Command);
+            cd.FrontendManager.AddMessageToChat(cd.Chat, builder.ToMessage());
         }
         
         public void UpdateNetworkStatus()
@@ -880,6 +948,18 @@ namespace Smuxi.Engine
             return chat;
         }
 
+        public PersonChatModel CreatePersonChat(PersonModel person,
+                                                IProtocolManager protocolManager)
+        {
+            Trace.Call(person, protocolManager);
+
+            if (person == null) {
+                throw new ArgumentNullException("person");
+            }
+            return CreatePersonChat(person, person.ID, person.IdentityName,
+                                    protocolManager);
+        }
+
         public PersonChatModel CreatePersonChat(PersonModel person,
                                                 string id, string name,
                                                 IProtocolManager protocolManager)
@@ -1011,11 +1091,13 @@ namespace Smuxi.Engine
             }
         }
 
+        [Obsolete("This method is deprecated, use AddMessageToChat(ChatModel, MessageModel) instead!")]
         public void AddTextToChat(ChatModel chat, string text)
         {
             AddTextToChat(chat, text, false);
         }
 
+        [Obsolete("This method is deprecated, use AddMessageToChat(ChatModel, MessageModel, bool) instead!")]
         public void AddTextToChat(ChatModel chat, string text, bool ignoreFilters)
         {
             if (chat == null) {
@@ -1103,6 +1185,7 @@ namespace Smuxi.Engine
 #if LOG4NET
             f_Logger.Debug("AddPersonToGroupChat() groupChat.Name: "+groupChat.Name+" person.IdentityName: "+person.IdentityName);
 #endif
+            groupChat.UnsafePersons.Add(person.ID, person);
             lock (_FrontendManagers) {
                 foreach (FrontendManager fm in _FrontendManagers.Values) {
                     fm.AddPersonToGroupChat(groupChat, person);
@@ -1131,8 +1214,8 @@ namespace Smuxi.Engine
             // FIXME: do we have to lock groupChat.UnsafePersons here?
             // probably not, as long as the ProtocolManager who owns this chat
             // is only running one thread
-            groupChat.UnsafePersons.Remove(oldPerson.ID.ToLower());
-            groupChat.UnsafePersons.Add(newPerson.ID.ToLower(), newPerson);
+            groupChat.UnsafePersons.Remove(oldPerson.ID);
+            groupChat.UnsafePersons.Add(newPerson.ID, newPerson);
             lock (_FrontendManagers) {
                 foreach (FrontendManager fm in _FrontendManagers.Values) {
                     fm.UpdatePersonInGroupChat(groupChat, oldPerson, newPerson);
@@ -1172,7 +1255,7 @@ namespace Smuxi.Engine
 #if LOG4NET
             f_Logger.Debug("RemovePersonFromGroupChat() groupChat.Name: " + groupChat.Name + " person.ID: "+person.ID);
 #endif
-            groupChat.UnsafePersons.Remove(person.ID.ToLower());
+            groupChat.UnsafePersons.Remove(person.ID);
             lock (_FrontendManagers) {
                 foreach (FrontendManager fm in _FrontendManagers.Values) {
                     fm.RemovePersonFromGroupChat(groupChat, person);
