@@ -35,7 +35,7 @@ namespace Smuxi.Frontend.Gnome
             }
             
             TransientFor = parent;
-            Name = Frontend.Name;
+            ProgramName = Frontend.Name;
             var version = Frontend.Version.ToString();
             var distVersion = Defines.DistVersion;
             if (!String.IsNullOrEmpty(distVersion)) {
@@ -50,20 +50,27 @@ namespace Smuxi.Frontend.Gnome
                 "Clément Bourgeois <moonpyk@gmail.com>",
                 "Chris Le Sueur <c.m.lesueur@gmail.com>",
                 "Tuukka Hastrup <Tuukka.Hastrup@iki.fi>",
-                "Bianca Mix <heavydemon@freenet.de>"
+                "Bianca Mix <heavydemon@freenet.de>",
+                "Oliver Schneider <mail@oli-obk.de>",
+                "Carlos Martín Nieto <cmn@dwim.me>"
             };
             Artists = new string[] {
                 "Jakub Steiner <jimmac@ximian.com>",
                 "Rodney Dawes <dobey@novell.com>",
                 "Lapo Calamandrei <calamandrei@gmail.com>",
-                "Ahmed Abdellah <a3dman1@gmail.com>"
+                "Ahmed Abdellah <a3dman1@gmail.com>",
+                "Matthieu James <matthieu.james@gmail.com>",
+                "George Karavasilev <motorslav@gmail.com>"
             };
             TranslatorCredits = _("translator-credits");
             Logo = Frontend.LoadIcon(
                 Frontend.IconName, 256, "icon_256x256.png"
             );
-            Website = "http://www.smuxi.org/";
-            WebsiteLabel = _("Smuxi Website");
+            // HACK: shows "not implemented" error on OS X
+            if (!Frontend.IsMacOSX) {
+                Website = "http://www.smuxi.org/";
+                WebsiteLabel = _("Smuxi Website");
+            }
         }
         
         private static string _(string msg)
