@@ -1,7 +1,7 @@
 /*
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2005-2011 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2005-2013 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -95,7 +95,7 @@ namespace Smuxi.Frontend.Gnome
         public bool IsSensitive {
             set {
                 f_Notebook.Sensitive = value;
-                Frontend.MainWindow.MenuBar.Sensitive = value;
+                Frontend.MainWindow.MenuWidget.Sensitive = value;
                 Frontend.MainWindow.Entry.Sensitive = value;
             }
             get {
@@ -138,6 +138,7 @@ namespace Smuxi.Frontend.Gnome
 
             f_Notebook.RemovePage(f_Notebook.PageNum(chatView));
             f_Chats.Remove(chatView);
+            SyncManager.Remove(chat);
             SyncedChats.Remove(chatView);
 
             if (ChatRemoved != null) {
@@ -215,13 +216,32 @@ namespace Smuxi.Frontend.Gnome
         {
             Trace.Call();
 
+            var chats = new List<ChatView>(f_Chats);
+            foreach (var chat in chats) {
+                // clean up ChatView and all notify related resources like
+                // StatusIconManager, NotifyManager and IndicateManager
+                RemoveChat(chat.ChatModel);
+            }
+
             f_Config = null;
-            f_Chats.Clear();
-            f_Notebook.RemoveAllPages();
             SyncedChats.Clear();
             SyncManager.Clear();
         }
 
+        public void ClearAllActivity()
+        {
+            Trace.Call();
+
+            f_Notebook.ClearAllActivity();
+        }
+
+        public void Minimize()
+        {
+            Trace.Call();
+
+            Frontend.MainWindow.Iconify();
+        }
+
         void OnChatAdded(object sender, ChatViewAddedEventArgs e)
         {
             Trace.Call(sender, e);
@@ -233,6 +253,7 @@ namespace Smuxi.Frontend.Gnome
                 chatView.ID = e.ChatID;
                 chatView.Name = e.ChatID;
                 chatView.Position = e.ChatPosition;
+                chatView.ProtocolManager = e.ProtocolManager;
                 f_Chats.Add(chatView);
 
                 if (f_Config != null) {
