@@ -1,7 +1,7 @@
 /*
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2005-2012 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2005-2013 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -43,6 +43,8 @@ namespace Smuxi.Frontend.Gnome
         private CommandManager   _CommandManager;
         private new EntrySettings Settings { get; set; }
 
+        private NickCompleter NickCompleter { get; set; }
+
         ChatViewManager ChatViewManager;
         event EventHandler<EventArgs> Activated;
 
@@ -243,6 +245,13 @@ namespace Smuxi.Frontend.Gnome
         
         protected virtual void ProcessKey(Gtk.KeyPressEventArgs e)
         {
+            if (Frontend.IsWindows && String.IsNullOrEmpty(Text)) {
+                // HACK: workaround rendering issue on Windows where the text
+                // cursor and first typed character are not showing up until
+                // a 2nd character is typed, see #810
+                QueueDraw();
+            }
+
             int keynumber = (int)e.Event.KeyValue;
             Gdk.Key key = e.Event.Key;
             if ((e.Event.State & Gdk.ModifierType.ControlMask) != 0) {
@@ -274,11 +283,20 @@ namespace Smuxi.Frontend.Gnome
                             ChatViewManager.CurrentChatNumber++;
                         }
                         break;
-                    // don't break unicode input
-                    case Gdk.Key.U:
-                    // don't break copy/paste
                     case Gdk.Key.c:
                     case Gdk.Key.C:
+                        // only use copy if something is selected in the entry
+                        if (Buffer.HasSelection) {
+                            e.RetVal = false;
+                            break;
+                        }
+                        // copy selection from main chat window
+                        var buf = ChatViewManager.CurrentChatView.OutputMessageTextView.Buffer;
+                        buf.CopyClipboard(Gtk.Clipboard.Get(Gdk.Selection.Clipboard));
+                        break;
+                    // don't break unicode input
+                    case Gdk.Key.U:
+                    // don't break paste
                     case Gdk.Key.v:
                     case Gdk.Key.V:
                     // don't break select all
@@ -297,6 +315,10 @@ namespace Smuxi.Frontend.Gnome
                     case Gdk.Key.End:
                         ChatViewManager.CurrentChatView.ScrollToEnd();
                         break;
+                    // anything else we let GTK+ handle
+                    default:
+                        e.RetVal = false;
+                        break;
                 }
             }
             
@@ -349,6 +371,30 @@ namespace Smuxi.Frontend.Gnome
                         pagenumber = 19;
                         break;
                 }
+                switch (key) {
+                    case Gdk.Key.h:
+                    case Gdk.Key.H:
+                        if (Frontend.IsMacOSX) {
+                            Frontend.MainWindow.Iconify();
+                            e.RetVal = true;
+                        }
+                        break;
+                    case Gdk.Key.braceleft:
+                    case Gdk.Key.Up:
+                        if (Frontend.IsMacOSX) {
+                            ChatViewManager.CurrentChatNumber--;
+                            e.RetVal = true;
+                        }
+                        break;
+                    case Gdk.Key.braceright:
+                    case Gdk.Key.Down:
+                        if (Frontend.IsMacOSX) {
+                            ChatViewManager.CurrentChatNumber++;
+                            e.RetVal = true;
+                        }
+                        break;
+                }
+
 
                 if (pagenumber != -1) {
                     ChatViewManager.CurrentChatNumber = pagenumber;
@@ -424,8 +470,9 @@ namespace Smuxi.Frontend.Gnome
                 }
                 
                 if (Text.IndexOf("\n") != -1) {
+                    var text = Text.TrimEnd('\n');
                     // seems to be a paste, so let's break it apart
-                    string[] msgParts = Text.Split(new char[] {'\n'});
+                    string[] msgParts = text.Split(new char[] {'\n'});
                     if (msgParts.Length > 3) {
                         string msg = String.Format(_("You are going to paste {0} lines. Do you want to continue?"),
                                                    msgParts.Length);
@@ -442,8 +489,13 @@ namespace Smuxi.Frontend.Gnome
                             return;
                         }
                     }
-                    foreach (string msg in msgParts) {
-                        ExecuteCommand(msg);
+                    if (Frontend.EngineVersion < new Version(0,8,11)) {
+                        foreach (string msg in msgParts) {
+                            ExecuteCommand(msg);
+                        }
+                    } else {
+                        // new engines know how to handle messages containing \n
+                        ExecuteCommand(text);
                     }
                 } else {
                     ExecuteCommand(Text);
@@ -503,10 +555,6 @@ namespace Smuxi.Frontend.Gnome
                         _CommandEcho(cd);
                         handled = true;
                         break;
-                    case "exec":
-                        _CommandExec(cd);
-                        handled = true;
-                        break;
                     case "window":
                         _CommandWindow(cd);
                         handled = true;
@@ -523,6 +571,14 @@ namespace Smuxi.Frontend.Gnome
                         _CommandSync(cd);
                         handled = true;
                         break;
+                    case "gc":
+                        GC.Collect();
+                        handled = true;
+                        break;
+                    case "generate_messages":
+                        CommandGenerateMessages(cd);
+                        handled = true;
+                        break;
                 }
             }
             
@@ -559,7 +615,7 @@ namespace Smuxi.Frontend.Gnome
 
         private void _CommandList(CommandModel cd)
         {
-            Frontend.MainWindow.OpenFindGroupChatWindow(cd.Parameter);
+            Frontend.OpenFindGroupChatWindow(cd.Parameter);
         }
 
         private void _CommandDetach(CommandModel cd)
@@ -569,38 +625,15 @@ namespace Smuxi.Frontend.Gnome
 
         private void _CommandEcho(CommandModel cd)
         {
-            cd.FrontendManager.AddTextToChat(
-                cd.Chat,
-                String.Format("-!- {0}", cd.Parameter)
-            );
-        }
-        
-        private void _CommandExec(CommandModel cd)
-        {
-            if (cd.DataArray.Length >= 2) {
-                string output;
-                System.Diagnostics.Process process = new System.Diagnostics.Process();
-                process.StartInfo.FileName = cd.DataArray[1];
-                if (cd.DataArray.Length >= 3) { 
-                    process.StartInfo.Arguments = String.Join(" ", cd.DataArray,
-                                                    2, cd.DataArray.Length-2);
-                }
-                process.StartInfo.RedirectStandardOutput = true;
-                process.StartInfo.RedirectStandardError = true;
-                process.StartInfo.UseShellExecute = false;
-                
-                try {
-                    process.Start();
-                    output = process.StandardOutput.ReadToEnd();
-                    cd.FrontendManager.AddTextToChat(cd.Chat, output);
-                } catch {
-                }
-            }
+            var msg = new MessageBuilder().
+                AppendEventPrefix().
+                AppendText(cd.Parameter).
+                ToMessage();
+            cd.FrontendManager.AddMessageToChat(cd.Chat, msg);
         }
     
         private void _CommandWindow(CommandModel cd)
         {
-            FrontendManager fm = cd.FrontendManager;
             if (cd.DataArray.Length >= 2) {
                 var currentChat = ChatViewManager.CurrentChatView;
                 if (cd.Parameter.ToLower() == "close") {
@@ -676,150 +709,58 @@ namespace Smuxi.Frontend.Gnome
             ChatViewManager.CurrentChatView.Clear();
         }
 
+        void CommandGenerateMessages(CommandModel cd)
+        {
+            var count = 0;
+            Int32.TryParse(cd.Parameter, out count);
+
+            var chat = ChatViewManager.CurrentChatView;
+            var builder = new MessageBuilder();
+            var sender = new ContactModel("msg-tester", "msg-tester", "test", "test");
+            builder.AppendMessage(sender, "time for a messsage generator command so I can test speed and memory usage");
+            var text = builder.CreateText(" *formatted text* ");
+            text.Bold = true;
+            builder.Append(text);
+            builder.AppendUrl("https://www.smuxi.org/");
+
+            var msgs = new List<MessageModel>(count);
+            for (var i = 0; i < count; i++) {
+                var msg = builder.ToMessage();
+                msgs.Add(msg);
+            }
+
+            DateTime start, stop;
+            start = DateTime.UtcNow;
+            foreach (var msg in msgs) {
+                chat.AddMessage(msg);
+            }
+            stop = DateTime.UtcNow;
+
+            builder = new MessageBuilder();
+            builder.AppendText(
+                "ChatView.AddMessage(): count: {0} took: {1:0} ms avg: {2:0.00} ms",
+                count,
+                (stop - start).TotalMilliseconds,
+                (stop - start).TotalMilliseconds / count
+            );
+            chat.AddMessage(builder.ToMessage());
+
+            builder = new MessageBuilder();
+            builder.AppendText(
+                "ChatView.AddMessage(): MessageTextTagTable.Size: {0}",
+                chat.OutputMessageTextView.MessageTextTagTable.Size
+            );
+            chat.AddMessage(builder.ToMessage());
+        }
+
         private void _NickCompletion()
         {
-            // return if we don't support the current ChatView
-            if (!(ChatViewManager.CurrentChatView is GroupChatView) &&
-                !(ChatViewManager.CurrentChatView is PersonChatView)) {
-                return;
-            }
-
-            int position = Position;
+            // perform completion
             string text = Text;
-            string word;
-            int previous_space;
-            int next_space;
-
-            // find the current word
-            string temp;
-            temp = text.Substring(0, position);
-            previous_space = temp.LastIndexOf(' ');
-            next_space = text.IndexOf(' ', position);
-
-#if LOG4NET
-            _Logger.Debug("previous_space: "+previous_space);
-            _Logger.Debug("next_space: "+next_space);
-#endif
-
-            if (previous_space != -1 && next_space != -1) {
-                // previous and next space exist
-                word = text.Substring(previous_space + 1, next_space - previous_space - 1);
-            } else if (previous_space != -1) {
-                // previous space exist
-                word = text.Substring(previous_space + 1);
-            } else if (next_space != -1) {
-                // next space exist
-                word = text.Substring(0, next_space);
-            } else {
-                // no spaces
-                word = text;
-            }
-
-            if (word == String.Empty) {
-                return;
-            }
-
-            bool leadingAt = false;
-            // remove leading @ character
-            if (word.StartsWith("@")) {
-                word = word.Substring(1);
-                leadingAt = true;
-            }
-
-            // find the possible nickname
-            bool found = false;
-            bool partial_found = false;
-            string nick = null;
-
-            ChatView view = ChatViewManager.CurrentChatView;
-            if (view is GroupChatView) {
-                GroupChatView cp = (GroupChatView) view;
-                if (Settings.BashStyleCompletion) {
-                    IList<string> result = cp.PersonLookupAll(word);
-                    if (result == null || result.Count == 0) {
-                        // no match
-                    } else if (result.Count == 1) {
-                        found = true;
-                        nick = result[0];
-                    } else if (result.Count >= 2) {
-                        string[] nickArray = new string[result.Count];
-                        result.CopyTo(nickArray, 0);
-                        string nicks = String.Join(" ", nickArray, 1, nickArray.Length - 1);
-                        ChatViewManager.CurrentChatView.AddMessage(
-                            new MessageModel(String.Format("-!- {0}", nicks))
-                        );
-                        found = true;
-                        partial_found = true;
-                        nick = result[0];
-                    }
-                } else {
-                    PersonModel person = cp.PersonLookup(word);
-                    if (person != null) {
-                        found = true;
-                        nick = person.IdentityName;
-                     }
-                }
-            } else {
-                var personChat = (PersonChatView) ChatViewManager.CurrentChatView;
-                var personModel = personChat.PersonModel;
-                if (personModel.IdentityName.StartsWith(word,
-                        StringComparison.InvariantCultureIgnoreCase)) {
-                    found = true;
-                    nick = personModel.IdentityName;
-                }
-            }
-
-            string completionChar = Settings.CompletionCharacter;
-            // add leading @ back and supress completion character
-            if (leadingAt) {
-                word = String.Format("@{0}", word);
-                nick = String.Format("@{0}", nick);
-                completionChar = String.Empty;
-            }
-
-            if (found) {
-                // put the found nickname in place
-                if (previous_space != -1 && next_space != -1) {
-                    // previous and next space exist
-                    temp = text.Remove(previous_space + 1, word.Length);
-                    temp = temp.Insert(previous_space + 1, nick);
-                    Text = temp;
-                    if (partial_found) {
-                        Position = previous_space + 1 + nick.Length;
-                    } else {
-                        Position = previous_space + 2 + nick.Length;
-                    }
-                } else if (previous_space != -1) {
-                    // only previous space exist
-                    temp = text.Remove(previous_space + 1, word.Length);
-                    temp = temp.Insert(previous_space + 1, nick);
-                    if (partial_found) {
-                        Text = temp;
-                    } else {
-                        Text = temp + " ";
-                    }
-                    Position = previous_space + 2 + nick.Length;
-                } else if (next_space != -1) {
-                    // only next space exist
-                    temp = text.Remove(0, next_space + 1);
-                    if (partial_found) {
-                        Text = nick + " " + temp;
-                        Position = nick.Length;
-                    } else {
-                        Text = String.Format("{0}{1} {2}", nick,
-                                             completionChar, temp);
-                        Position = nick.Length + completionChar.Length + 1;
-                    }
-                } else {
-                    // no spaces
-                    if (partial_found) {
-                        Text = nick;
-                    } else {
-                        Text = String.Format("{0}{1} ", nick, completionChar);
-                    }
-                    Position = -1;
-                }
-            }
+            int position = Position;
+            NickCompleter.Complete(ref text, ref position, ChatViewManager.CurrentChatView);
+            Text = text;
+            Position = position;
         }
         
         public virtual void ApplyConfig(UserConfig config)
@@ -844,6 +785,16 @@ namespace Smuxi.Frontend.Gnome
             ModifyFont(theme.FontDescription);
 
             Settings.ApplyConfig(config);
+
+            // replace nick completer if needed
+            if (Settings.BashStyleCompletion && !(NickCompleter is LongestPrefixNickCompleter)) {
+                NickCompleter = new LongestPrefixNickCompleter();
+            } else if (!Settings.BashStyleCompletion && !(NickCompleter is TabCycleNickCompleter)) {
+                NickCompleter = new TabCycleNickCompleter();
+            }
+
+            // set the completion character
+            NickCompleter.CompletionChar = Settings.CompletionCharacter;
         }
 
         private void InitCommandManager()
@@ -883,12 +834,12 @@ namespace Smuxi.Frontend.Gnome
         }
 
 #if GTKSPELL
-        [DllImport("gtkspell.dll")]
+        [DllImport("gtkspell.dll", CallingConvention = CallingConvention.Cdecl)]
         static extern IntPtr gtkspell_new_attach(IntPtr text_view,
                                                  string locale,
                                                  IntPtr error);
 
-        [DllImport("gtkspell.dll")]
+        [DllImport("gtkspell.dll", CallingConvention = CallingConvention.Cdecl)]
         static extern void gtkspell_detach(IntPtr obj);
 #endif
     }
