@@ -26,6 +26,7 @@ using System.Linq;
 using System.Threading;
 using System.Reflection;
 using SysDiag = System.Diagnostics;
+using MonoDevelop.MacInterop;
 using Smuxi.Engine;
 using Smuxi.Common;
 
@@ -41,14 +42,10 @@ namespace Smuxi.Frontend.Gnome
         private static readonly string    _UIName = "GNOME";
         private static int                _UIThreadID;
         private static Version            _Version;
-        private static string             _VersionNumber;
         private static string             _VersionString;
         private static Version            _EngineVersion;
         private static SplashScreenWindow _SplashScreenWindow;
         private static MainWindow         _MainWindow;
-#if GTK_SHARP_2_10
-        private static StatusIconManager  _StatusIconManager;
-#endif
         private static FrontendConfig     _FrontendConfig;
         private static Session            _LocalSession;
         private static Session            _Session;
@@ -62,6 +59,11 @@ namespace Smuxi.Frontend.Gnome
         public static string IconName { get; private set; }
         public static bool HasSystemIconTheme { get; private set; }
         public static bool HadSession { get; private set; }
+        public static bool IsGtkInitialized { get; private set; }
+        public static bool InGtkApplicationRun { get; private set; }
+        public static bool IsWindows { get; private set; }
+        public static bool IsUnity { get; private set; }
+        public static bool IsMacOSX { get; private set; }
 
         public static event EventHandler  SessionPropertyChanged;
 
@@ -172,6 +174,19 @@ namespace Smuxi.Frontend.Gnome
             }
         }
 
+        static Frontend()
+        {
+            IsWindows = Environment.OSVersion.Platform == PlatformID.Win32NT;
+            IsMacOSX = Platform.OperatingSystem == "Darwin";
+            var desktop = Environment.GetEnvironmentVariable("XDG_CURRENT_DESKTOP");
+            if (!String.IsNullOrEmpty(desktop) && desktop.ToLower().Contains("unity")) {
+#if LOG4NET
+                _Logger.Debug("Frontend(): Detected Unity desktop envrionment");
+#endif
+                IsUnity = true;
+            }
+        }
+
         public static void Init(string[] args)
         {
             System.Threading.Thread.CurrentThread.Name = "Main";
@@ -181,7 +196,6 @@ namespace Smuxi.Frontend.Gnome
             AssemblyProductAttribute pr = (AssemblyProductAttribute)asm.
                 GetCustomAttributes(typeof(AssemblyProductAttribute), false)[0];
             _Version = asm_name.Version;
-            _VersionNumber = asm_name.Version.ToString();
             _VersionString = pr.Product + " - " + _UIName + " frontend " + _Version;
 
 #if LOG4NET
@@ -232,8 +246,39 @@ namespace Smuxi.Frontend.Gnome
             if (_SplashScreenWindow != null) {
                 _SplashScreenWindow.Destroy();
             }
-            
+
+            if (IsMacOSX) {
+                ApplicationEvents.Quit += delegate(object sender, ApplicationQuitEventArgs e) {
+                    Quit();
+                    e.Handled = true;
+                };
+
+                ApplicationEvents.Reopen += delegate(object sender, ApplicationEventArgs e) {
+                    MainWindow.Deiconify();
+                    MainWindow.Visible = true;
+                    e.Handled = true;
+                };
+
+                ApplicationEvents.OpenUrls += delegate(object sender, ApplicationUrlEventArgs e) {
+                    e.Handled = true;
+                    if (e.Urls == null || e.Urls.Count == 0) {
+                        return;
+                    }
+                    foreach (var url in e.Urls) {
+                        try {
+                            OpenChatLink(new Uri(url));
+                        } catch (Exception ex) {
+#if LOG4NET
+                            _Logger.Error("ApplicationEvents.OpenUrls() Exception", ex);
+#endif
+                        }
+                    }
+                };
+            }
+
+            InGtkApplicationRun = true;
             Gtk.Application.Run();
+            InGtkApplicationRun = false;
 #if LOG4NET
             _Logger.Warn("Gtk.Application.Run() returned!");
 #endif
@@ -709,11 +754,16 @@ namespace Smuxi.Frontend.Gnome
         }
 
         public static void OpenChatLink(Uri link)
+        {
+            TryOpenChatLink(link);
+        }
+
+        public static bool TryOpenChatLink(Uri link)
         {
             Trace.Call(link);
 
             if (Session == null) {
-                return;
+                return false;
             }
 
             // supported:
@@ -777,6 +827,15 @@ namespace Smuxi.Frontend.Gnome
             }
 
             if (manager == null) {
+                // only irc may autoconnect to a server
+                switch (linkProtocol) {
+                    case "irc":
+                    case "ircs":
+                    case "smuxi":
+                        break;
+                    default:
+                        return false;
+                }
                 ServerModel server = null;
                 if (!String.IsNullOrEmpty(linkNetwork)) {
                     // try to find a server with this network name and connect to it
@@ -797,7 +856,7 @@ namespace Smuxi.Frontend.Gnome
             }
 
             if (String.IsNullOrEmpty(linkChat)) {
-                return;
+                return true;
             }
 
             // switch to existing chat
@@ -807,7 +866,7 @@ namespace Smuxi.Frontend.Gnome
                 }
                 if (String.Compare(chatView.ID, linkChat, true) == 0) {
                     MainWindow.ChatViewManager.CurrentChatView = chatView;
-                    return;
+                    return true;
                 }
             }
 
@@ -822,6 +881,81 @@ namespace Smuxi.Frontend.Gnome
                     }
                 });
             }
+            return true;
+        }
+
+        public static void OpenLink(Uri link)
+        {
+            Trace.Call(link);
+
+            if (link == null) {
+                throw new ArgumentNullException("link");
+            }
+
+            if (TryOpenChatLink(link)) {
+                return;
+            }
+
+            // hopefully MS .NET / Mono finds some way to handle the URL
+            ThreadPool.QueueUserWorkItem(delegate {
+                try {
+                    var url = link.ToString();
+                    using (var process = SysDiag.Process.Start(url)) {
+                        // Start() might return null in case it re-used a
+                        // process instead of starting one
+                        if (process != null) {
+                            process.WaitForExit();
+                        }
+                    }
+                } catch (Exception ex) {
+                    // exceptions in the thread pool would kill the process, see:
+                    // http://msdn.microsoft.com/en-us/library/0ka9477y.aspx
+                    // http://projects.qnetp.net/issues/show/194
+#if LOG4NET
+                    _Logger.Error("OpenLink(): opening URL: '" + link + "' failed", ex);
+#endif
+                }
+            });
+        }
+
+        public static void OpenFindGroupChatWindow()
+        {
+            OpenFindGroupChatWindow(null);
+        }
+
+        public static void OpenFindGroupChatWindow(string searchKey)
+        {
+            var chatView = MainWindow.ChatViewManager.CurrentChatView;
+            if (chatView == null) {
+                return;
+            }
+
+            var manager = chatView.ProtocolManager;
+            if (manager == null) {
+                return;
+            }
+
+            var dialog = new FindGroupChatDialog(
+                MainWindow, manager
+            );
+            if (!String.IsNullOrEmpty(searchKey)) {
+                dialog.NameEntry.Text = searchKey;
+                dialog.FindButton.Click();
+            }
+            var res = dialog.Run();
+            var groupChat = dialog.GroupChat;
+            dialog.Destroy();
+            if (res != (int) Gtk.ResponseType.Ok) {
+                return;
+            }
+
+            ThreadPool.QueueUserWorkItem(delegate {
+                try {
+                    manager.OpenChat(Frontend.FrontendManager, groupChat);
+                } catch (Exception ex) {
+                    Frontend.ShowException(null, ex);
+                }
+            });
         }
 
 #if GTK_SHARP_2_10
@@ -905,6 +1039,7 @@ namespace Smuxi.Frontend.Gnome
             _Logger.Debug("InitGtk(): Using locale data from: " + localeDir);
 #endif
             Gtk.Application.Init(Name, ref args);
+            IsGtkInitialized = true;
 #if GTK_SHARP_2_10
             GLib.ExceptionManager.UnhandledException += _OnUnhandledException;
 #endif
@@ -913,6 +1048,7 @@ namespace Smuxi.Frontend.Gnome
             var iconPath = Path.Combine(Defines.InstallPrefix, "share");
             iconPath = Path.Combine(iconPath, "icons");
             var theme = Gtk.IconTheme.Default;
+            var settings = Gtk.Settings.Default;
             var iconInfo = theme.LookupIcon(IconName, -1, 0);
             HasSystemIconTheme = iconInfo != null &&
                                  iconInfo.Filename != null &&
@@ -922,6 +1058,44 @@ namespace Smuxi.Frontend.Gnome
                                 HasSystemIconTheme ? "system" : "built-in");
 #endif
 
+            var unityWithLightIcons = false;
+            if (Frontend.IsUnity) {
+                var sysGtkTheme = settings.ThemeName ?? String.Empty;
+                var sysIconTheme = GetGtkIconThemeName() ?? String.Empty;
+#if LOG4NET
+                _Logger.DebugFormat("InitGtk(): Detected GTK+ theme: {0} " +
+                                    "icon theme: {1}", sysGtkTheme,
+                                    sysIconTheme);
+#endif
+                if (sysGtkTheme.StartsWith("Ambiance") &&
+                    sysIconTheme != "ubuntu-mono-dark") {
+#if LOG4NET
+                    _Logger.Debug("InitGtk(): Detected Ambiance theme with "+
+                                  "light icons");
+#endif
+                    unityWithLightIcons = true;
+                }
+            }
+            var appIconDir = Path.Combine(appDir, "icons");
+            if (Directory.Exists(appIconDir) &&
+                (Frontend.IsMacOSX ||
+                 Frontend.IsWindows ||
+                 unityWithLightIcons)) {
+                var iconTheme = "Faenza-Smuxi";
+#if LOG4NET
+                _Logger.InfoFormat("InitGtk(): Setting icon theme to: {0}",
+                                    iconTheme);
+#endif
+                settings.SetStringProperty(
+                    "gtk-icon-theme-name", iconTheme, Assembly.GetExecutingAssembly().FullName
+                );
+#if LOG4NET
+                _Logger.InfoFormat("InitGtk(): Prepending {0} to icon search path",
+                                    appIconDir);
+#endif
+                theme.PrependSearchPath(appIconDir);
+            }
+
             if (HasSystemIconTheme) {
                 Gtk.Window.DefaultIconName = "smuxi-frontend-gnome";
             } else {
@@ -935,15 +1109,23 @@ namespace Smuxi.Frontend.Gnome
         {
             // HACK: Force GTK# to use the right GTK+ install as the PATH
             // environment variable might contain other GTK+ installs
+            // GTK# 2.12.20
             var installPath = (string) Microsoft.Win32.Registry.GetValue(
-                "HKEY_LOCAL_MACHINE\\SOFTWARE\\Novell\\GtkSharp\\InstallFolder",
+                "HKEY_LOCAL_MACHINE\\SOFTWARE\\Xamarin\\GtkSharp\\InstallFolder",
                 "", null
             );
             if (installPath == null) {
+                // GTK# 2.12.10
+                installPath = (string) Microsoft.Win32.Registry.GetValue(
+                    "HKEY_LOCAL_MACHINE\\SOFTWARE\\Novell\\GtkSharp\\InstallFolder",
+                    "", null
+                );
+            }
+            if (installPath == null) {
 #if LOG4NET
                 _Logger.Error("InitGtkPathWin(): couldn't obtain GTK# installation folder from registry. GTK# is probably incorrectly installed!");
-                return;
 #endif
+                return;
             }
 
             var binPath = Path.Combine(installPath, "bin");
@@ -987,6 +1169,26 @@ namespace Smuxi.Frontend.Gnome
             }
         }
 
+        static string GetGtkIconThemeName()
+        {
+            // HACK: Gtk.IconTheme is not exposing gtk-icon-theme-name
+            var method = typeof(Gtk.Settings).GetMethod(
+                "GetProperty",
+                BindingFlags.Instance | BindingFlags.NonPublic
+            );
+            if (method == null) {
+#if LOG4NET
+                _Logger.Warn("GetGtkIconThemeName(): method is null!");
+#endif
+                return String.Empty;
+            }
+            var value = (string)(GLib.Value) method.Invoke(
+                Gtk.Settings.Default,
+                new object[] {"gtk-icon-theme-name"}
+            );
+            return value;
+        }
+
         private static string _(string msg)
         {
             return Mono.Unix.Catalog.GetString(msg);
