@@ -118,6 +118,12 @@ namespace Smuxi.Frontend.Gnome
             _Glade.Autoconnect(this);
             _Dialog = (Gtk.Dialog)_Glade["PreferencesDialog"];
             _Dialog.TransientFor = parent;
+            if (Frontend.IsMacOSX) {
+                // HACK: center on parent is not working for this dialog
+                _Dialog.SetPosition(Gtk.WindowPosition.CenterAlways);
+            } else {
+                _Dialog.SetPosition(Gtk.WindowPosition.CenterOnParent);
+            }
             
             ((Gtk.Button)_Glade["OKButton"]).Clicked += new EventHandler(_OnOKButtonClicked);
             ((Gtk.Button)_Glade["ApplyButton"]).Clicked += new EventHandler(_OnApplyButtonClicked);
@@ -277,7 +283,12 @@ namespace Smuxi.Frontend.Gnome
 
             _Load();
         }
-        
+
+        public void Show()
+        {
+            _Dialog.Show();
+        }
+
         private void _Load()
         {
             Trace.Call();
@@ -839,7 +850,7 @@ namespace Smuxi.Frontend.Gnome
             foreach (string word in highlight_words) {
                 if (word.StartsWith("/") && word.EndsWith("/")) {
                     try {
-                        Regex regex = new Regex(word.Substring(1, word.Length - 2));
+                        new Regex(word.Substring(1, word.Length - 2));
                     } catch (ArgumentException ex) {
                         throw new ApplicationException(
                             String.Format(
