@@ -1,7 +1,7 @@
 /*
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2005-2011 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2005-2013 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -21,6 +21,7 @@
  */
 
 using System;
+using System.Text.RegularExpressions;
 using System.Threading;
 using System.Collections.Generic;
 using Smuxi.Common;
@@ -59,6 +60,7 @@ namespace Smuxi.Frontend.Gnome
         bool                         UseLowBandwidthMode { get; set; }
         protected Gtk.Image          TabImage { get; set; }
         bool                         IsAutoScrolling { get; set; }
+        Gtk.ImageMenuItem  CloseItem { get; set; }
 
         public ChatModel ChatModel {
             get {
@@ -214,6 +216,13 @@ namespace Smuxi.Frontend.Gnome
             }
         }
 
+        // by default: no participants
+        public virtual IList<PersonModel> Participants {
+            get {
+                return new List<PersonModel>();
+            }
+        }
+
         public Gtk.Widget LabelWidget {
             get {
                 return _TabEventBox;
@@ -238,6 +247,12 @@ namespace Smuxi.Frontend.Gnome
             }
         }
 
+        protected Gtk.Menu TabMenu {
+            get {
+                return _TabMenu;
+            }
+        }
+
         protected ThemeSettings ThemeSettings {
             get {
                 return _ThemeSettings;
@@ -270,6 +285,7 @@ namespace Smuxi.Frontend.Gnome
             tv.SizeRequested += delegate {
                 AutoScroll();
             };
+            tv.PersonClicked += OnMessageTextViewPersonClicked;
             _OutputMessageTextView = tv;
 
             Gtk.ScrolledWindow sw = new Gtk.ScrolledWindow();
@@ -278,19 +294,17 @@ namespace Smuxi.Frontend.Gnome
             sw.HscrollbarPolicy = Gtk.PolicyType.Automatic;
             sw.VscrollbarPolicy = Gtk.PolicyType.Always;
             sw.ShadowType = Gtk.ShadowType.In;
-            sw.Vadjustment.ValueChanged += delegate {
-                CheckAutoScroll();
-            };
+            sw.Vadjustment.ValueChanged += OnVadjustmentValueChanged;
             sw.Add(_OutputMessageTextView);
 
             // popup menu
             _TabMenu = new Gtk.Menu();
-            
-            Gtk.ImageMenuItem close_item = new Gtk.ImageMenuItem(Gtk.Stock.Close, null);
-            close_item.Activated += new EventHandler(OnTabMenuCloseActivated);  
-            _TabMenu.Append(close_item);
+
+            CloseItem = new Gtk.ImageMenuItem(Gtk.Stock.Close, null);
+            CloseItem.Activated += new EventHandler(OnTabMenuCloseActivated);
+            _TabMenu.Append(CloseItem);
             _TabMenu.ShowAll();
-            
+
             //FocusChild = _OutputTextView;
             //CanFocus = false;
             
@@ -328,10 +342,8 @@ namespace Smuxi.Frontend.Gnome
         {
             Trace.Call();
 
-            base.Dispose();
-
             Dispose(true);
-            GC.SuppressFinalize(this);
+            base.Dispose();
         }
 
         protected void Dispose(bool disposing)
@@ -343,6 +355,12 @@ namespace Smuxi.Frontend.Gnome
                     _LastSeenHighlightQueue.Dispose();
                 }
                 _LastSeenHighlightQueue = null;
+
+                // HACK: this shouldn't be needed but GTK# keeps GC handles
+                // these callbacks for some reason and thus leaks :(
+                _OutputMessageTextView.Dispose();
+                CloseItem.Activated -= OnTabMenuCloseActivated;
+                _OutputScrolledWindow.Vadjustment.ValueChanged -= OnVadjustmentValueChanged;
             }
         }
 
@@ -466,8 +484,6 @@ namespace Smuxi.Frontend.Gnome
 
             // REMOTING CALL
             SyncedName = _ChatModel.Name;
-            // REMOTING CALL
-            ProtocolManager = _ChatModel.ProtocolManager;
 
             if (!Frontend.IsLocalEngine && Frontend.UseLowBandwidthMode) {
                 // FIXME: set TabImage back to normal
@@ -744,6 +760,22 @@ namespace Smuxi.Frontend.Gnome
             popup.ShowAll();
         }
 
+        protected virtual void OnMessageTextViewPersonClicked(object sender, MessageTextViewPersonClickedEventArgs e)
+        {
+            Trace.Call(sender, e);
+
+            var entry = Frontend.MainWindow.Entry;
+            var text = entry.Text;
+            var match = Regex.Match(text, "^[^ ]+: ");
+            if (match.Success) {
+                // removing existing nick
+                text = text.Substring(match.Length);
+            }
+            text = String.Format("{0}: {1}", e.IdentityName, text);
+            entry.Text = text;
+            entry.HasFocus = true;
+        }
+
         protected virtual void OnLastSeenHighlightQueueExceptionEvent(object sender, TaskQueueExceptionEventArgs e)
         {
             Trace.Call(sender, e);
@@ -764,6 +796,11 @@ namespace Smuxi.Frontend.Gnome
 #endif
         }
 
+        void OnVadjustmentValueChanged(object sender, EventArgs e)
+        {
+            CheckAutoScroll();
+        }
+
         private static string _(string msg)
         {
             return Mono.Unix.Catalog.GetString(msg);
