@@ -1,7 +1,7 @@
 /*
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2005-2006, 2009-2011 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2005-2006, 2009-2013 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -39,7 +39,6 @@ namespace Smuxi.Frontend.Gnome
         public static Gdk.Pixbuf   IconPixbuf { get; private set; }
         private bool               NickColors { get; set; }
         private GroupChatModel     _GroupChatModel;
-        private Gtk.ScrolledWindow _PersonScrolledWindow;
         private Gtk.TreeView       _PersonTreeView;
         private Gtk.ListStore      _PersonListStore;
         private Gtk.Menu           _PersonMenu;
@@ -83,6 +82,20 @@ namespace Smuxi.Frontend.Gnome
             }
         }
 
+        public override IList<PersonModel> Participants {
+            get {
+                var participants = new List<PersonModel>();
+                if (_PersonListStore == null) {
+                    return participants;
+                }
+                foreach (object[] row in _PersonListStore) {
+                    var person = (PersonModel) row[0];
+                    participants.Add(person);
+                }
+                return participants;
+            }
+        }
+
         protected Gtk.TreeView PersonTreeView {
             get {
                 return _PersonTreeView;
@@ -124,7 +137,6 @@ namespace Smuxi.Frontend.Gnome
             _OutputHPaned = new Gtk.HPaned();
             
             Gtk.ScrolledWindow sw = new Gtk.ScrolledWindow();
-            _PersonScrolledWindow = sw;
             //sw.WidthRequest = 150;
             sw.HscrollbarPolicy = Gtk.PolicyType.Never;
             
@@ -143,6 +155,7 @@ namespace Smuxi.Frontend.Gnome
             column.Spacing = 0;
             column.SortIndicator = false;
             column.Sizing = Gtk.TreeViewColumnSizing.Autosize;
+            // FIXME: this callback leaks memory
             column.SetCellDataFunc(cellr, new Gtk.TreeCellDataFunc(RenderPersonIdentityName));
             tv.AppendColumn(column);
             _IdentityNameColumn = column;
@@ -153,6 +166,17 @@ namespace Smuxi.Frontend.Gnome
             _PersonListStore = liststore;
             
             tv.Model = liststore;
+            tv.SearchColumn = 0;
+            tv.SearchEqualFunc = (model, col, key, iter) => {
+                var person = (PersonModel) model.GetValue(iter, col);
+                // Ladies and gentlemen welcome to C
+                // 0 means it matched but 0 as bool is false. So if it matches
+                // we have to return false. Still not clear? true is false and
+                // false is true, weirdo! If you think this is retarded,
+                // yes it is.
+                return !person.IdentityName.StartsWith(key, StringComparison.InvariantCultureIgnoreCase);
+            };
+            tv.EnableSearch = true;
             tv.RowActivated += new Gtk.RowActivatedHandler(OnPersonsRowActivated);
             tv.FocusOutEvent += OnPersonTreeViewFocusOutEvent;
             
@@ -179,9 +203,7 @@ namespace Smuxi.Frontend.Gnome
             _TopicTextView.WrapMode = Gtk.WrapMode.WordChar;
             _TopicScrolledWindow = new Gtk.ScrolledWindow();
             _TopicScrolledWindow.ShadowType = Gtk.ShadowType.In;
-            // when using PolicyType.Never, it will try to grow but never shrinks!
-            _TopicScrolledWindow.HscrollbarPolicy = Gtk.PolicyType.Automatic;
-            _TopicScrolledWindow.VscrollbarPolicy = Gtk.PolicyType.Automatic;
+            _TopicScrolledWindow.HscrollbarPolicy = Gtk.PolicyType.Never;
             _TopicScrolledWindow.Add(_TopicTextView);
             // make sure the topic is invisible and remains by default and
             // visible when a topic gets set
@@ -190,14 +212,22 @@ namespace Smuxi.Frontend.Gnome
             _TopicScrolledWindow.NoShowAll = true;
             _TopicScrolledWindow.SizeRequested += delegate(object o, Gtk.SizeRequestedArgs args) {
                 // predict and set useful topic heigth
-                Pango.Layout layout = _TopicTextView.CreatePangoLayout("Test Topic");
                 int lineWidth, lineHeigth;
-                layout.GetPixelSize(out lineWidth, out lineHeigth);
+                using (var layout = _TopicTextView.CreatePangoLayout("Test Topic")) {
+                    layout.GetPixelSize(out lineWidth, out lineHeigth);
+                }
                 var lineSpacing = _TopicTextView.PixelsAboveLines +
                                   _TopicTextView.PixelsBelowLines;
-                var text = Topic != null ? Topic.ToString() : String.Empty;
-                // hardcoded to 2 lines for now
-                var newLines = text.Length > 0 ? 2 : 0;
+                var it = _TopicTextView.Buffer.StartIter;
+                int newLines = 1;
+                // move to end of next visual line
+                while (_TopicTextView.ForwardDisplayLineEnd(ref it)) {
+                    newLines++;
+                    // calling ForwardDisplayLineEnd repeatedly stays on the same position
+                    // therefor we move one cursor position further
+                    it.ForwardCursorPosition();
+                }
+                newLines = Math.Min(newLines, 3);
                 var bestSize = new Gtk.Requisition() {
                     Height = ((lineHeigth + lineSpacing) * newLines) + 2
                 };
@@ -210,7 +240,24 @@ namespace Smuxi.Frontend.Gnome
             
             ShowAll();
         }
-        
+
+        public override void Dispose()
+        {
+            Trace.Call();
+
+            // HACK: this shouldn't be needed but GTK# keeps GC handles
+            // these callbacks for some reason and thus leaks :(
+            // release ListStore.SetSortFunc() callback
+            // gtk_list_store_finalize() -> _gtk_tree_data_list_header_free() -> destroy(user_data);
+            _TopicTextView.Dispose();
+            _PersonListStore.Dispose();
+            // release TreeViewColumn.SetCellDataFunc() callback
+            // gtk_tree_view_column_finalize -> GtkTreeViewColumnCellInfo -> info->destroy(info->func_data)
+            _IdentityNameColumn.Dispose();
+
+            base.Dispose();
+        }
+
         public override void Disable()
         {
             Trace.Call();
@@ -278,6 +325,7 @@ namespace Smuxi.Frontend.Gnome
                 // BUG? TreeView doesn't seem to recognize existing values in the model?!?
                 // see: http://www.smuxi.org/issues/show/132
                 _PersonTreeView.Model = ls;
+                _PersonTreeView.SearchColumn = 0;
                 
                 /*
                 // predict and set useful width
@@ -294,6 +342,7 @@ namespace Smuxi.Frontend.Gnome
                 status += String.Format(" {0}", _("done."));
                 Frontend.MainWindow.Status = status;
             }
+            SyncedPersons = null;
 
             Topic = SyncedTopic;
 
@@ -376,90 +425,6 @@ namespace Smuxi.Frontend.Gnome
             UpdatePersonCount();
         }
 
-        /*Logic taken from the PersonLookup method of Engine/Chats/GroupChatModel.cs */
-        public PersonModel PersonLookup(string identityName)
-        {
-            Trace.Call(identityName);
-#if LOG4NET
-            _Logger.Debug("PersonLookup(): GroupChatModel.Name: " + Name);
-#endif
-            int identityNameLength = identityName.Length;
-            if (_PersonListStore == null) {
-                return null;
-            }
-            foreach (object[] row in _PersonListStore) {
-                PersonModel person = (PersonModel)row[0];
-                if ((person.IdentityName.Length >= identityNameLength) &&
-                    (person.IdentityName.Substring(0, identityNameLength).ToLower() == identityName.ToLower())) {
-#if LOG4NET
-                    _Logger.Debug("PersonLookup(): found: " + person.IdentityName);
-#endif
-                    return person;
-                }
-            }
-
-#if LOG4NET
-            _Logger.Debug("PersonLookup() no matching identityName found");
-#endif
-            return null;
-        }
-
-        /*Logic taken from the PersonLookupAll method of Engine/Chats/GroupChatModel.cs */
-        public IList<string> PersonLookupAll(string identityName)
-        {
-            Trace.Call(identityName);
-
-            IList<string> foundIdentityNames = new List<string>();
-            int identityNameLength = identityName.Length;
-            string longestIdentityName = String.Empty;
-            if (_PersonListStore == null) {
-                return foundIdentityNames;
-            }
-            foreach (object[] row in _PersonListStore) {
-                PersonModel person = (PersonModel)row[0];
-                if ((person.IdentityName.Length >= identityNameLength) &&
-                    (person.IdentityName.Substring(0, identityNameLength).ToLower() == identityName.ToLower())) {
-                    foundIdentityNames.Add(person.IdentityName);
-                    if (person.IdentityName.Length > longestIdentityName.Length) {
-                        longestIdentityName = person.IdentityName;
-                    }
-                }
-            }
-
-            // guess the common part of the found nicknames
-            string common_nick = identityName;
-            bool match = true;
-            while (match) {
-                if (common_nick.Length >= longestIdentityName.Length) {
-                    break;
-                }
-                common_nick += longestIdentityName[common_nick.Length];
-                foreach (string name in foundIdentityNames) {
-                    if (!name.ToLower().StartsWith(common_nick.ToLower())) {
-                        common_nick = common_nick.Substring(0, common_nick.Length - 1);
-                        match = false;
-                     }
-                }
-            }
-
-            if (foundIdentityNames.Count == 0) {
-#if LOG4NET
-                _Logger.Debug("PersonLookupAll(): no matching identityName found");
-#endif
-            } else if (foundIdentityNames.Count == 1) {
-#if LOG4NET
-                _Logger.Debug("PersonLookupAll(): found exact match: " + foundIdentityNames[0]);
-#endif
-            } else {
-#if LOG4NET
-                _Logger.Debug("PersonLookupAll(): found " + foundIdentityNames.Count + " matches");
-#endif
-                foundIdentityNames.Insert(0, common_nick);
-            }
-            return foundIdentityNames;
-        }
-
-        
         public override void ApplyConfig(UserConfig config)
         {
             Trace.Call(config);
