@@ -1,7 +1,7 @@
 /*
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2005-2006, 2009-2011 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2005-2006, 2009-2013 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -49,6 +49,7 @@ namespace Smuxi.Frontend.Gnome
         public string Host { get; private set; }
         public int Port { get; private set; }
         public string NetworkID { get; private set; }
+        Gtk.ImageMenuItem  ReconnectItem { get; set; }
 
         protected override Gtk.Image DefaultTabImage {
             get {
@@ -81,6 +82,7 @@ namespace Smuxi.Frontend.Gnome
             NetworkWebsiteUrls.Add("GSDnet", "http://www.gsd-software.net/");
             NetworkWebsiteUrls.Add("ustream", "http://www.ustream.tv/");
             NetworkWebsiteUrls.Add("Infinity-IRC", "http://www.infinityirc.com/");
+            NetworkWebsiteUrls.Add("GeekShed", "http://www.geekshed.net/");
 
             // Twitter
             NetworkWebsiteUrls.Add("Twitter", "http://www.twitter.com/");
@@ -88,6 +90,12 @@ namespace Smuxi.Frontend.Gnome
             // XMPP
             NetworkWebsiteUrls.Add("talk.google.com", "http://www.google.com/talk/");
             NetworkWebsiteUrls.Add("chat.facebook.com", "http://www.facebook.com/");
+
+            // JabbR
+            NetworkWebsiteUrls.Add("jabbr.net", "http://jabbr.net/");
+
+            // Campfire
+            NetworkWebsiteUrls.Add("Campfire", "http://campfirenow.com");
         }
 
         public ProtocolChatView(ChatModel chat) : base(chat)
@@ -97,6 +105,13 @@ namespace Smuxi.Frontend.Gnome
             ProxySettings = new ProxySettings();
 
             Add(OutputScrolledWindow);
+
+            ReconnectItem = new Gtk.ImageMenuItem(_("Reconnect"));
+            ReconnectItem.Image = new Gtk.Image(Gtk.Stock.Refresh, Gtk.IconSize.Menu);
+            ReconnectItem.Activated += new EventHandler(OnTabMenuReconnectActivated);
+            TabMenu.Prepend(ReconnectItem);
+            TabMenu.ShowAll();
+
             ShowAll();
         }
 
@@ -197,8 +212,9 @@ namespace Smuxi.Frontend.Gnome
 
             string websiteUrl = null;
             lock (NetworkWebsiteUrls) {
-                if (!NetworkWebsiteUrls.TryGetValue(ID, out websiteUrl)) {
-                    // unknown network, nothing to download
+                if (!NetworkWebsiteUrls.TryGetValue(ID, out websiteUrl) &&
+                    !NetworkWebsiteUrls.TryGetValue(protocol, out websiteUrl)) {
+                    // unknown network and protocol, nothing to download
                     return;
                 }
                 // download in background so Sync() doesn't get slowed down
@@ -328,6 +344,28 @@ namespace Smuxi.Frontend.Gnome
             });
         }
 
+        protected virtual void OnTabMenuReconnectActivated(object sender, EventArgs e)
+        {
+            Trace.Call(sender, e);
+
+            try {
+                var pm = ProtocolManager;
+                if (pm == null) {
+                    return;
+                }
+
+                ThreadPool.QueueUserWorkItem(delegate {
+                    try {
+                        pm.Reconnect(Frontend.FrontendManager);
+                    } catch (Exception ex) {
+                        Frontend.ShowException(ex);
+                    }
+                });
+            } catch (Exception ex) {
+                Frontend.ShowException(ex);
+            }
+        }
+
         static bool ValidateCertificate(object sender,
                                         X509Certificate certificate,
                                         X509Chain chain,
