@@ -1,7 +1,7 @@
 /*
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2009-2011 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2009-2013 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -38,6 +38,7 @@ namespace Smuxi.Frontend.Gnome
 #endif        
         private static readonly Gdk.Cursor _NormalCursor = new Gdk.Cursor(Gdk.CursorType.Xterm);
         private static readonly Gdk.Cursor _LinkCursor = new Gdk.Cursor(Gdk.CursorType.Hand2);
+        static bool IsGtk2_17 { get; set; }
         private Gtk.TextTagTable _MessageTextTagTable;
         private MessageModel _LastMessage;
         private bool         _ShowTimestamps;
@@ -45,7 +46,6 @@ namespace Smuxi.Frontend.Gnome
         private bool         _ShowMarkerline;
         private bool         _AtLinkTag;
         private Uri          _ActiveLink;
-        private UserConfig   _Config;
         private ThemeSettings _ThemeSettings;
         private Gdk.Color    _MarkerlineColor = new Gdk.Color(255, 0, 0);
         private int          _MarkerlineBufferPosition;
@@ -57,8 +57,12 @@ namespace Smuxi.Frontend.Gnome
         Gtk.TextTag LinkTag { get; set; }
         Gtk.TextTag EventTag { get; set; }
 
+        Gtk.TextTag PersonTag { get; set; }
+        bool AtPersonTag { get; set; }
+
         public event MessageTextViewMessageAddedEventHandler       MessageAdded;
         public event MessageTextViewMessageHighlightedEventHandler MessageHighlighted;
+        public event EventHandler<MessageTextViewPersonClickedEventArgs> PersonClicked;
         
         public bool ShowTimestamps {
             get {
@@ -132,6 +136,12 @@ namespace Smuxi.Frontend.Gnome
             }
         }
 
+        static MessageTextView()
+        {
+            IsGtk2_17 = String.IsNullOrEmpty(Gtk.Global.CheckVersion(2, 17, 0)) &&
+                        !String.IsNullOrEmpty(Gtk.Global.CheckVersion(2, 18, 0));
+        }
+
         public MessageTextView()
         {
             Trace.Call();
@@ -241,9 +251,11 @@ namespace Smuxi.Frontend.Gnome
             var startMark = new Gtk.TextMark(null, true);
             buffer.AddMark(startMark, iter);
 
+
             var senderPrefixWidth = GetSenderPrefixWidth(msg);
             Gtk.TextTag indentTag = null;
             if (senderPrefixWidth != 0) {
+                // TODO: re-use text tags that have the same indent width
                 indentTag = new Gtk.TextTag(null) {
                     Indent = -senderPrefixWidth
                 };
@@ -255,22 +267,20 @@ namespace Smuxi.Frontend.Gnome
                 if (_LastMessage != null) {
                     var lastMsgTimeStamp = _LastMessage.TimeStamp.ToLocalTime();
                     var span = msgTimeStamp.Date - lastMsgTimeStamp.Date;
-                    string dayLine = null;
-                    if (span.Days > 1) {
-                        dayLine = String.Format(
-                            "-!- " + _("Day changed from {0} to {1}"),
-                            lastMsgTimeStamp.ToShortDateString(),
-                            msgTimeStamp.ToShortDateString()
-                        );
-                    } else if (span.Days > 0) {
-                        dayLine = String.Format(
-                            "-!- " + _("Day changed to {0}"),
-                            msgTimeStamp.ToLongDateString()
-                        );
-                    }
-
-                    if (dayLine != null) {
-                        buffer.Insert(ref iter, dayLine + "\n");
+                    if (span.Days > 0) {
+                        var dayLine = new MessageBuilder().
+                            AppendEventPrefix();
+                        if (span.Days > 1) {
+                            dayLine.AppendText(_("Day changed from {0} to {1}"),
+                                               lastMsgTimeStamp.ToShortDateString(),
+                                               msgTimeStamp.ToShortDateString());
+                        } else {
+                            dayLine.AppendText(_("Day changed to {0}"),
+                                               msgTimeStamp.ToLongDateString());
+                        }
+                        dayLine.AppendText("\n");
+                        var dayLineMsg = dayLine.ToMessage().ToString();
+                        Buffer.InsertWithTags(ref iter, dayLineMsg, EventTag);
                     }
                 }
                 
@@ -295,6 +305,9 @@ namespace Smuxi.Frontend.Gnome
                 }
             }
 
+            var msgStartMark = new Gtk.TextMark(null, true);
+            buffer.AddMark(msgStartMark, iter);
+
             bool hasHighlight = false;
             foreach (MessagePartModel msgPart in msg.MessageParts) {
                 // supposed to be used only in a ChatView
@@ -389,17 +402,34 @@ namespace Smuxi.Frontend.Gnome
             if (indentTag != null) {
                 buffer.ApplyTag(indentTag, startIter, iter);
             }
+            var nick = msg.GetNick();
+            if (nick != null) {
+                // TODO: re-use the same person tag for the same nick
+                var personTag = new PersonTag(nick, nick);
+                personTag.TextEvent += OnPersonTagTextEvent;
+                _MessageTextTagTable.Add(personTag);
+
+                var msgStartIter = buffer.GetIterAtMark(msgStartMark);
+                var nickEndIter = msgStartIter;
+                nickEndIter.ForwardChars(nick.Length + 2);
+                buffer.ApplyTag(PersonTag, msgStartIter, nickEndIter);
+                buffer.ApplyTag(personTag, msgStartIter, nickEndIter);
+            }
+            buffer.DeleteMark(startMark);
+            buffer.DeleteMark(msgStartMark);
             if (addLinebreak) {
                 buffer.Insert(ref iter, "\n");
             }
 
             CheckBufferSize();
 
-            // HACK: force a redraw of the widget, as for some reason
-            // GTK+ 2.17.6 is not redrawing some lines we add here, especially
-            // for local messages. See:
-            // http://projects.qnetp.net/issues/show/185
-            QueueDraw();
+            if (IsGtk2_17) {
+                // HACK: force a redraw of the widget, as for some reason
+                // GTK+ 2.17.6 is not redrawing some lines we add here, especially
+                // for local messages. See:
+                // http://projects.qnetp.net/issues/show/185
+                QueueDraw();
+            }
 
             if (MessageAdded != null) {
                 MessageAdded(this, new MessageTextViewMessageAddedEventArgs(msg));
@@ -426,6 +456,21 @@ namespace Smuxi.Frontend.Gnome
             QueueDraw();
         }
 
+        public override void Dispose()
+        {
+            // HACK: this shouldn't be needed but GTK# keeps GC handles
+            // these callbacks for some reason and thus leaks :(
+            _MessageTextTagTable.Foreach(tag => {
+                if (tag is LinkTag) {
+                    tag.TextEvent -= OnLinkTagTextEvent;
+                } else if (tag is PersonTag) {
+                    tag.TextEvent -= OnPersonTagTextEvent;
+                }
+            });
+            _MessageTextTagTable.Dispose();
+            base.Dispose();
+        }
+
         /*
          * Helper methods
          */
@@ -470,6 +515,10 @@ namespace Smuxi.Frontend.Gnome
             LinkTag = tt;
             ttt.Add(tt);
 
+            tt = new Gtk.TextTag("person");
+            PersonTag = tt;
+            ttt.Add(tt);
+
             return ttt;
         }
 
@@ -489,11 +538,16 @@ namespace Smuxi.Frontend.Gnome
             // get TextIter with buffer position
             Gtk.TextIter iter = GetIterAtLocation(bufferX, bufferY);
             bool atUrlTag = false;
+            bool atPersonTag = false;
             foreach (Gtk.TextTag tag in iter.Tags) {
                 if (tag.Name == "link") {
                     atUrlTag = true;
                     break;
                 }
+                if (tag.Name == "person") {
+                    atPersonTag = true;
+                    break;
+                }
             }
             
             Gdk.Window window = GetWindow(Gtk.TextWindowType.Text); 
@@ -513,6 +567,21 @@ namespace Smuxi.Frontend.Gnome
                     _ActiveLink = null;
                 }
             }
+            if (atPersonTag != AtPersonTag) {
+                AtPersonTag = atPersonTag;
+                
+                if (atPersonTag) {
+#if LOG4NET
+                    _Logger.Debug("OnMotionNotifyEvent(): at person tag");
+#endif
+                    window.Cursor = _LinkCursor;
+                } else {
+#if LOG4NET
+                    _Logger.Debug("OnMotionNotifyEvent(): not at person tag");
+#endif
+                    window.Cursor = _NormalCursor;
+                }
+            }
         }
         
         protected virtual void OnLinkTagTextEvent(object sender, Gtk.TextEventArgs e)
@@ -542,7 +611,37 @@ namespace Smuxi.Frontend.Gnome
                 return;
             }
 
-            OpenLink(_ActiveLink);
+            Frontend.OpenLink(_ActiveLink);
+        }
+
+        protected virtual void OnPersonTagTextEvent(object sender, Gtk.TextEventArgs e)
+        {
+            // logging noise
+            //Trace.Call(sender, e);
+            
+            // if something in the textview is selected, bail out
+            if (HasTextViewSelection) {
+#if LOG4NET
+                _Logger.Debug("OnPersonTagTextEvent(): active selection present, bailing out...");
+#endif
+                return;
+            }
+            
+            var tag = (PersonTag) sender;
+            if (tag == null) {
+                return;
+            }
+
+            if (e.Event.Type != Gdk.EventType.ButtonPress) {
+                return;
+            }
+
+            if (PersonClicked != null) {
+                PersonClicked(
+                    this,
+                    new MessageTextViewPersonClickedEventArgs(tag.IdentityName)
+                );
+            }
         }
 
         protected virtual void OnPopulatePopup(object sender, Gtk.PopulatePopupArgs e)
@@ -562,7 +661,7 @@ namespace Smuxi.Frontend.Gnome
             Gtk.ImageMenuItem open_item = new Gtk.ImageMenuItem(Gtk.Stock.Open, null);
             open_item.Activated += delegate {
                 if (_ActiveLink != null) {
-                    OpenLink(_ActiveLink);
+                    Frontend.OpenLink(_ActiveLink);
                 }
             };
             popup.Append(open_item);
@@ -581,36 +680,6 @@ namespace Smuxi.Frontend.Gnome
             popup.ShowAll();
         }
 
-        private void OpenLink(Uri link)
-        {
-            Trace.Call(link);
-
-            if (link == null) {
-                throw new ArgumentNullException("link");
-            }
-
-            // hopefully MS .NET / Mono finds some way to handle the URL
-            ThreadPool.QueueUserWorkItem(delegate {
-                var url = link.ToString();
-                try {
-                    using (var process = SysDiag.Process.Start(url)) {
-                        // Start() might return null in case it re-used a
-                        // process instead of starting one
-                        if (process != null) {
-                            process.WaitForExit();
-                        }
-                    }
-                } catch (Exception ex) {
-                    // exceptions in the thread pool would kill the process, see:
-                    // http://msdn.microsoft.com/en-us/library/0ka9477y.aspx
-                    // http://projects.qnetp.net/issues/show/194
-#if LOG4NET
-                    _Logger.Error("OpenLink(): opening URL: '" + url + "' failed", ex);
-#endif
-                }
-            });
-        }
-        
         private string GetTextTagName(TextColor fgColor, TextColor bgColor)
         {
              string hexcode;
@@ -675,14 +744,45 @@ namespace Smuxi.Frontend.Gnome
                 return;
             }
 
-            if (Buffer.LineCount > _BufferLines) {
-                Gtk.TextIter start_iter = Buffer.StartIter;
+            var buffer = Buffer;
+            if (buffer.LineCount > _BufferLines) {
+                Gtk.TextIter start_iter = buffer.StartIter;
                 // TODO: maybe we should delete chunks instead of each line
-                Gtk.TextIter end_iter = Buffer.GetIterAtLine(Buffer.LineCount -
+                Gtk.TextIter end_iter = buffer.GetIterAtLine(buffer.LineCount -
                                                              _BufferLines);
                 int offset = end_iter.Offset;
-                Buffer.Delete(ref start_iter, ref end_iter);
-                // TODO: remove unnamed tags from TextTagTable
+
+                // release tags
+                var toggled_tags = new List<Gtk.TextTag>(16);
+                var start_tags = start_iter.GetToggledTags(true);
+                toggled_tags.AddRange(start_tags);
+
+                var tag_iter = start_iter;
+                while (tag_iter.ForwardToTagToggle(null)) {
+                    if (tag_iter.Compare(end_iter) >= 0) {
+                        // tag is after line end
+                        break;
+                    }
+                    var iter_tags = tag_iter.GetToggledTags(true);
+                    toggled_tags.AddRange(iter_tags);
+                }
+
+                foreach (var tag in toggled_tags) {
+                    // don't remove color tags as they are shared wither other lines
+                    var tagName = tag.Name;
+                    if (tagName != null &&
+                        (tagName.StartsWith("fg_color:") ||
+                         tagName.StartsWith("bg_color:"))) {
+                        continue;
+                    }
+                    if (tag.IndentSet || tag is LinkTag) {
+                        buffer.RemoveTag(tag, start_iter, end_iter);
+                        _MessageTextTagTable.Remove(tag);
+                        tag.Dispose();
+                    }
+                }
+
+                buffer.Delete(ref start_iter, ref end_iter);
 
                 // update markerline offset if present
                 if (_MarkerlineBufferPosition != 0) {
@@ -745,15 +845,15 @@ namespace Smuxi.Frontend.Gnome
 
         int GetPangoWidth(string text, bool isMarkup)
         {
-            Pango.Layout layout;
-            if (isMarkup) {
-                layout = CreatePangoLayout(null);
-                layout.SetMarkup(text);
-            } else {
-                layout = CreatePangoLayout(text);
-            }
             int width, heigth;
-            layout.GetPixelSize(out width, out heigth);
+            using (var layout = CreatePangoLayout(null)) {
+                if (isMarkup) {
+                    layout.SetMarkup(text);
+                } else {
+                    layout.SetText(text);
+                }
+                layout.GetPixelSize(out width, out heigth);
+            }
             return width;
         }
 
@@ -798,4 +898,14 @@ namespace Smuxi.Frontend.Gnome
             f_Message = message;
         }
     }
+
+    public class MessageTextViewPersonClickedEventArgs : EventArgs
+    {
+        public string IdentityName { get; private set; }
+        
+        public MessageTextViewPersonClickedEventArgs(string identityName)
+        {
+            IdentityName = identityName;
+        }
+    }
 }
