@@ -1,7 +1,7 @@
 /*
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2007, 2010-2011 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2007, 2010-2013 Mirco Bauer <meebey@meebey.net>
  * Copyright (c) 2011 Andrius Bentkus <andrius.bentkus@gmail.com>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
@@ -38,6 +38,9 @@ namespace Smuxi.Frontend.Stfl
 #endif
         MainWindow      f_MainWindow;
         ChatViewManager f_ChatViewManager;
+        CommandManager CommandManager { get; set; }
+        NickCompleter NickCompleter { get; set; }
+        EntrySettings Settings { get; set; }
 
         event EventHandler Activated;
 
@@ -61,20 +64,62 @@ namespace Smuxi.Frontend.Stfl
         
         public Entry(MainWindow mainWindow, ChatViewManager chatViewManager)
         {
-           if (mainWindow == null) {
+            if (mainWindow == null) {
                 throw new ArgumentNullException("mainWindow");
-           }
-           if (chatViewManager == null) {
+            }
+            if (chatViewManager == null) {
                 throw new ArgumentNullException("chatViewManager");
-           }
+            }
 
             f_MainWindow = mainWindow;
             f_MainWindow.KeyPressed += OnKeyPressed;
             
             f_ChatViewManager = chatViewManager;
-            f_ChatViewManager.CurrentChatSwitched += OnChatSwitched;
+
+            Frontend.SessionPropertyChanged += delegate {
+                InitCommandManager();
+            };
+
+            Settings = new EntrySettings();
+            NickCompleter = new TabCycleNickCompleter();
         }
         
+        public virtual void ApplyConfig(UserConfig config)
+        {
+            Trace.Call(config);
+
+            if (config == null) {
+                throw new ArgumentNullException("config");
+            }
+
+            Settings.ApplyConfig(config);
+
+            // replace nick completer if needed
+            if (Settings.BashStyleCompletion && !(NickCompleter is LongestPrefixNickCompleter)) {
+                NickCompleter = new LongestPrefixNickCompleter();
+            } else if (!Settings.BashStyleCompletion && !(NickCompleter is TabCycleNickCompleter)) {
+                NickCompleter = new TabCycleNickCompleter();
+            }
+
+            // set the completion character
+            NickCompleter.CompletionChar = Settings.CompletionCharacter;
+        }
+
+        void InitCommandManager()
+        {
+            Trace.Call();
+
+            if (CommandManager != null) {
+                CommandManager.Dispose();
+            }
+
+            if (Frontend.Session == null) {
+                CommandManager = null;
+            } else {
+                CommandManager = new CommandManager(Frontend.Session);
+            }
+        }
+
         private void OnKeyPressed(object sender, KeyPressedEventArgs e)
         {
             Trace.Call(sender, e);
@@ -96,6 +141,9 @@ namespace Smuxi.Frontend.Stfl
                         f_ChatViewManager.ActiveChat.ScrollDown();
                     }
                     break;
+                case "TAB":
+                    CompleteNick();
+                    break;
                 case "kPRV5": // CTRL + PAGE UP
                 case "^P":
                     f_ChatViewManager.CurrentChatNumber--;
@@ -119,14 +167,6 @@ namespace Smuxi.Frontend.Stfl
             }
         }
 
-        private void OnChatSwitched(object sender, ChatSwitchedEventArgs e)
-        {
-            Trace.Call(sender, e);
-
-            f_MainWindow.InputLabel = String.Format("[{0}]",
-                                                    e.ChatView.ChatModel.Name);
-        }
-
         public virtual void OnActivated(EventArgs e)
         {
             var text = Text;
@@ -150,29 +190,23 @@ namespace Smuxi.Frontend.Stfl
             }
 
             ChatModel chat = null;
-            if (f_MainWindow.ChatViewManager.ActiveChat != null) {
-                chat = f_MainWindow.ChatViewManager.ActiveChat.ChatModel;
+            var currentChat = f_ChatViewManager.CurrentChat;
+            if (currentChat != null) {
+                chat = currentChat.ChatModel;
             }
-            bool handled = false;
-            CommandModel cd = new CommandModel(Frontend.FrontendManager, chat,
-                                               (string)Frontend.UserConfig["Interface/Entry/CommandCharacter"],
-                                               cmd);
-            handled = Command(cd);
-            if (!handled) {
-                handled = Frontend.Session.Command(cd);
-            }
-            if (!handled) {
-                // we may have no network manager yet
-                Engine.IProtocolManager nm = Frontend.FrontendManager.CurrentProtocolManager;
-                if (nm != null) {
-                    handled = nm.Command(cd);
-                } else {
-                    handled = false;
-                }
-            }
-            if (!handled) {
-               _CommandUnknown(cd);
+            CommandModel cd = new CommandModel(
+                Frontend.FrontendManager,
+                chat,
+                (string) Frontend.UserConfig["Interface/Entry/CommandCharacter"],
+                cmd
+            );
+
+            var handled = Command(cd);
+            if (handled) {
+                return;
             }
+
+            CommandManager.Execute(cd);
         }
 
         private bool Command(CommandModel cmd)
@@ -195,8 +229,11 @@ namespace Smuxi.Frontend.Stfl
 #if LOG4NET
                         _Logger.Debug("GC.Collect()");
 #endif
-                        cmd.FrontendManager.AddTextToChat(cmd.Chat,
-                            "-!- GCing...");
+                        var msg = new MessageBuilder().
+                            AppendEventPrefix().
+                            AppendText("GCing...").
+                            ToMessage();
+                        cmd.FrontendManager.AddMessageToChat(cmd.Chat, msg);
                         GC.Collect();
                         handled = true;
                         break;
@@ -216,7 +253,7 @@ namespace Smuxi.Frontend.Stfl
 
             string[] help = {
                 "help",
-                "window number",
+                "window (number|close)",
                 "exit",
             };
 
@@ -230,6 +267,11 @@ namespace Smuxi.Frontend.Stfl
 
         private void CommandWindow(CommandModel cmd)
         {
+            if (cmd.Parameter == "close") {
+                f_ChatViewManager.CurrentChat.Close();
+                return;
+            }
+
             int window;
             if (!Int32.TryParse(cmd.Parameter, out window)) {
                 return;
@@ -241,12 +283,13 @@ namespace Smuxi.Frontend.Stfl
             f_ChatViewManager.CurrentChat = chat;
         }
 
-        private void _CommandUnknown(CommandModel cd)
+        private void CommandUnknown(CommandModel cmd)
         {
-            cd.FrontendManager.AddTextToChat(cd.Chat, "-!- " +
-                                String.Format(Catalog.GetString(
-                                              "Unknown Command: {0}"),
-                                              cd.Command));
+            var msg = new MessageBuilder().
+                AppendEventPrefix().
+                AppendText(_("Unknown Command: {0}"), cmd.Command).
+                ToMessage();
+            cmd.FrontendManager.AddMessageToChat(cmd.Chat, msg);
         }
 
         // gets the position of the first space left
@@ -335,6 +378,16 @@ namespace Smuxi.Frontend.Stfl
                    Text.Substring(Math.Min(Position + 1, Text.Length));
         }
 
+        void CompleteNick()
+        {
+            // perform completion
+            string text = Text;
+            int position = Position;
+            NickCompleter.Complete(ref text, ref position, f_ChatViewManager.CurrentChat);
+            Text = text;
+            Position = position;
+        }
+
         static string _(string msg)
         {
             return Mono.Unix.Catalog.GetString(msg);
