@@ -1,7 +1,7 @@
 /*
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2007, 2010-2011 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2007, 2010-2013 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -32,24 +32,9 @@ namespace Smuxi.Frontend.Stfl
 {
     public class MainWindow : Form
     {
-#if LOG4NET
-        private static readonly log4net.ILog _Logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
-#endif
-        private StflUI          _UI;
-        private Entry           _Entry;
-        private ChatViewManager _ChatViewManager;
-        
-        public ChatViewManager ChatViewManager {
-            get {
-                return _ChatViewManager;
-            }
-        }
-        
-        public StflUI UI {
-            get {
-                return _UI;
-            }
-        }
+        public StflUI UI { get; private set; }
+        Entry Entry { get; set; }
+        public ChatViewManager ChatViewManager { get; private set; }
 
         public string InputLabel {
             get {
@@ -89,16 +74,21 @@ namespace Smuxi.Frontend.Stfl
 
         public MainWindow() : base(null, "MainWindow.stfl")
         {
-            _ChatViewManager = new ChatViewManager(this);
-            _Entry = new Entry(this, _ChatViewManager);
-            _UI = new StflUI(_ChatViewManager);
+            ChatViewManager = new ChatViewManager(this);
+            Entry = new Entry(this, ChatViewManager);
+            UI = new StflUI(ChatViewManager);
 
             if (StflApi.IsXterm) {
                 ShowTitle = false;
             }
 
-    	    Assembly asm = Assembly.GetExecutingAssembly();
-    	    _ChatViewManager.Load(asm);
-    	}
+            Assembly asm = Assembly.GetExecutingAssembly();
+            ChatViewManager.Load(asm);
+        }
+
+        public void ApplyConfig(UserConfig config)
+        {
+            Entry.ApplyConfig(config);
+        }
     }
 }
