@@ -30,9 +30,6 @@ namespace Stfl
 {
     internal class StflApi
     {
-#if LOG4NET
-        private static readonly log4net.ILog f_Logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
-#endif
         public static bool IsXterm { get; private set; }
         static bool IsUtf8Locale { get; set; }
         static string EscapeLessThanCharacter  { get; set; }
@@ -40,7 +37,11 @@ namespace Stfl
 
         static StflApi()
         {
-            IsXterm = Environment.GetEnvironmentVariable("TERM") == "xterm";
+            // check if he has a graphical terminal. screen/tmux in not
+            // detected in case someone is using it in pure text mode
+            string termName = Environment.GetEnvironmentVariable("TERM");
+            IsXterm = (termName != null && (termName.StartsWith("xterm") ||
+                termName.StartsWith("rxvt")));
             // detect UTF-8 locale according to:
             // http://www.cl.cam.ac.uk/~mgk25/unicode.html#activate
             var locale = Environment.GetEnvironmentVariable("LC_ALL") ??
